import { Color, ImageAsset, Background, ImageSource, path as nsPath, knownFolders, Application } from '@nativescript/core';
import { isNumber, isString, isNullOrUndefined } from '@nativescript/core/utils/types';
import * as common from './common';
import { filterProperty, ImageCacheItBase, overlayColorProperty } from './common';
let ImageLoadedListener;
function initializeImageLoadedListener() {
    if (ImageLoadedListener) {
        return;
    }
    let ImageLoadedListenerImpl = class ImageLoadedListenerImpl extends java.lang.Object {
        constructor(owner) {
            super();
            this.owner = owner;
            return global.__native(this);
        }
        onImageLoaded(success) {
            const owner = this.owner;
            if (owner) {
                owner.isLoading = false;
            }
        }
    };
    ImageLoadedListenerImpl = __decorate([
        Interfaces([org.nativescript.widgets.image.Worker.OnImageLoadedListener]),
        __metadata("design:paramtypes", [Object])
    ], ImageLoadedListenerImpl);
    ImageLoadedListener = ImageLoadedListenerImpl;
}
export class ImageCacheIt extends ImageCacheItBase {
    constructor() {
        super();
        this.emptyBackground = new Background();
    }
    createNativeView() {
        return new com.github.triniwiz.imagecacheit.ImageView(this._context, null);
    }
    // nativeView: com.github.triniwiz.imagecacheit.ImageView;
    static get maxDiskCacheSize() {
        return com.github.triniwiz.imagecacheit.MyAppGlideModule.getMaxDiskCacheSize();
    }
    static set maxDiskCacheSize(size) {
        com.github.triniwiz.imagecacheit.MyAppGlideModule.setMaxDiskCacheSize(size);
    }
    static set maxMemoryCacheSize(size) {
        com.github.triniwiz.imagecacheit.MyAppGlideModule.setMaxMemoryCacheSize(size);
    }
    static get maxMemoryCacheSize() {
        return com.github.triniwiz.imagecacheit.MyAppGlideModule.getMaxMemoryCacheSize();
    }
    static get maxDiskCacheAge() {
        return com.github.triniwiz.imagecacheit.MyAppGlideModule.getMaxDiskCacheAge();
    }
    static set maxDiskCacheAge(age) {
        com.github.triniwiz.imagecacheit.MyAppGlideModule.setMaxDiskCacheAge(age);
    }
    initNativeView() {
        initializeImageLoadedListener();
        const nativeView = this.nativeViewProtected;
        const listener = new ImageLoadedListener(this);
        nativeView.setImageLoadedListener(listener);
        nativeView.listener = listener;
        const ref = new WeakRef(this);
        this._setOverlayColor(this.overlayColor);
        this.nativeView.setProgressListener(new com.github.triniwiz.imagecacheit.ProgressListener({
            onProgress(loaded, total, progress, url) {
                const owner = ref.get();
                if (owner) {
                    owner._emitProgressEvent(loaded, total, progress, url);
                }
            },
        }));
        this.nativeView.setEventsListener(new com.github.triniwiz.imagecacheit.EventsListener({
            onLoadStart() {
                const owner = ref.get();
                if (owner) {
                    owner._emitLoadStartEvent(owner.src);
                }
            },
            onLoadError(message) {
                const owner = ref.get();
                if (owner) {
                    owner._emitErrorEvent(message, owner.src);
                }
            },
            onLoadedEnd(image) {
                const owner = ref.get();
                if (owner) {
                    owner._emitLoadEndEvent(owner.src, image);
                }
            },
        }));
        this._setHeaders(this.headers);
        if (this.placeHolder) {
            ImageCacheIt._setPlaceHolder(this._context, this.placeHolder, this.nativeView);
        }
        if (this.errorHolder) {
            ImageCacheIt._setErrorHolder(this._context, this.errorHolder, this.nativeView);
        }
        if (this.fallback) {
            ImageCacheIt._setFallback(this._context, this.fallback, this.nativeView);
        }
        if (this.filter) {
            ImageCacheIt._setFilter(this.filter, this.nativeView);
        }
        if (this.stretch) {
            this._setStretch(this.stretch);
        }
        const image = ImageCacheIt.getImage(this._context, this.src);
        let decodeWidth = 0;
        let decodeHeight = 0;
        let keepAspectRatio = this._calculateKeepAspectRatio();
        if (isString(image) && this.nativeView) {
            this.nativeView.setSource(android.net.Uri.parse(image), decodeWidth, decodeHeight, keepAspectRatio, false, true);
        }
        else if (isNumber(image) || image instanceof java.lang.Integer) {
            this.nativeView.setSource(image, decodeWidth, decodeHeight, keepAspectRatio, false, true);
        }
        else if (image instanceof java.io.File) {
            this.nativeView.setSource(android.net.Uri.parse(image.getAbsolutePath()), decodeWidth, decodeHeight, keepAspectRatio, false, true);
        }
        else {
            if (this.src instanceof ImageAsset) {
                keepAspectRatio = !!this.src.options.keepAspectRatio;
            }
            this.nativeView.setSource(image, decodeWidth, decodeHeight, keepAspectRatio, false, true);
        }
    }
    _calculateKeepAspectRatio() {
        return this.stretch !== 'fill';
    }
    disposeNativeView() {
        super.disposeNativeView();
    }
    resetNativeView() {
        super.resetNativeView();
        this.nativeViewProtected.setImageMatrix(new android.graphics.Matrix());
    }
    [filterProperty.setNative](filter) {
        if (this.nativeView) {
            this.nativeView.setFilter(filter);
        }
    }
    _setOverlayColor(overlay) {
        if (!this.nativeViewProtected) {
            return;
        }
        if (overlay instanceof Color) {
            this.nativeViewProtected.setOverlayColor(overlay.android);
        }
        else if (typeof overlay === 'string') {
            this.nativeViewProtected.setOverlayColor(new Color(overlay).android);
        }
    }
    [overlayColorProperty.setNative](overlay) {
        this._setOverlayColor(overlay);
    }
    static isNumber(value) {
        return typeof value === 'number';
    }
    static getResourceId(context, res = '') {
        if (!context)
            return java.lang.Integer.valueOf(0);
        if (isString(res) && res.startsWith('res://')) {
            const packageName = context.getPackageName();
            try {
                const className = java.lang.Class.forName(`${packageName}.R$drawable`);
                return java.lang.Integer.valueOf(parseInt(String(className.getDeclaredField(res.replace('res://', '')).get(null))));
            }
            catch (e) {
                return java.lang.Integer.valueOf(0);
            }
        }
        return java.lang.Integer.valueOf(0);
    }
    static _setFallback(context, fallback, nativeView) {
        if (nativeView) {
            if (isString(fallback) && fallback.startsWith('res://')) {
                nativeView.setFallbackImage(fallback);
            }
            else {
                const holder = ImageCacheIt.getImage(context, fallback);
                nativeView.setFallbackImage(holder);
            }
        }
    }
    [common.fallbackProperty.setNative](fallback) {
        ImageCacheIt._setFallback(this._context, fallback, this.nativeView);
    }
    static _setPlaceHolder(context, placeHolder, nativeView) {
        if (nativeView) {
            if (isString(placeHolder) && placeHolder.startsWith('res://')) {
                nativeView.setPlaceHolder(placeHolder);
            }
            else {
                const holder = ImageCacheIt.getImage(context, placeHolder);
                nativeView.setPlaceHolder(holder);
            }
        }
    }
    [common.placeHolderProperty.setNative](placeHolder) {
        ImageCacheIt._setPlaceHolder(this._context, placeHolder, this.nativeView);
    }
    static _setErrorHolder(context, errorHolder, nativeView) {
        if (nativeView) {
            if (isString(errorHolder) && errorHolder.startsWith('res://')) {
                nativeView.setErrorHolder(errorHolder);
            }
            else {
                const holder = ImageCacheIt.getImage(context, errorHolder);
                nativeView.setErrorHolder(holder);
            }
        }
    }
    [common.errorHolderProperty.setNative](errorHolder) {
        ImageCacheIt._setErrorHolder(this._context, errorHolder, this.nativeView);
    }
    [common.srcProperty.getDefault]() {
        return undefined;
    }
    static _setSrc(context, src, nativeView, base) {
        const image = ImageCacheIt.getImage(context, src);
        if (nativeView) {
            let decodeWidth = 0;
            let decodeHeight = 0;
            let keepAspectRatio = base._calculateKeepAspectRatio();
            if (isString(image)) {
                nativeView.setSource(android.net.Uri.parse(image), decodeWidth, decodeHeight, keepAspectRatio, false, true);
            }
            else if (isNumber(image) || image instanceof java.lang.Integer) {
                nativeView.setSource(image, decodeWidth, decodeHeight, keepAspectRatio, false, true);
            }
            else if (image instanceof java.io.File) {
                nativeView.setSource(image, decodeWidth, decodeHeight, keepAspectRatio, false, true);
            }
            else {
                nativeView.setSource(image, decodeWidth, decodeHeight, keepAspectRatio, false, true);
            }
        }
    }
    [common.srcProperty.setNative](src) {
        ImageCacheIt._setSrc(this._context, src, this.nativeView, this);
    }
    [common.priorityProperty.getDefault]() {
        return common.Priority.Normal;
    }
    [common.priorityProperty.setNative](value) {
        if (!this.nativeView)
            return;
        switch (value) {
            case common.Priority.High:
                this.nativeView.setPriority(com.github.triniwiz.imagecacheit.ImageView.Priority.High);
                break;
            case common.Priority.Low:
                this.nativeView.setPriority(com.github.triniwiz.imagecacheit.ImageView.Priority.Low);
                break;
            default:
                this.nativeView.setPriority(com.github.triniwiz.imagecacheit.ImageView.Priority.Normal);
                break;
        }
    }
    [common.tintColorProperty.getDefault]() {
        return undefined;
    }
    [common.tintColorProperty.setNative](value) {
        if (!value) {
            this.nativeView.clearColorFilter();
        }
        else {
            this.nativeView.setColorFilter(value.android);
        }
    }
    [common.headersProperty.getDefault]() {
        return new Map();
    }
    _setHeaders(value) {
        const headers = new java.util.HashMap();
        if (value) {
            value.forEach((value, key) => {
                headers.put(key, value);
            });
        }
        if (this.nativeView) {
            this.nativeView.setHeaders(headers);
        }
    }
    [common.headersProperty.setNative](value) {
        this._setHeaders(value);
    }
    static getImage(context, src) {
        let nativeImage = null;
        if (isNullOrUndefined(src)) {
            return null;
        }
        if (isString(src)) {
            if (src.substr(0, 1) === '/') {
                nativeImage = new java.io.File(src);
            }
            else if (src.startsWith('~/')) {
                nativeImage = new java.io.File(nsPath.join(knownFolders.currentApp().path, src.replace('~/', '')));
            }
            else if (src.startsWith('http')) {
                nativeImage = src;
            }
            else if (src.startsWith('res://')) {
                nativeImage = this.getResourceId(context, src);
            }
        }
        else if (src instanceof ImageSource || src instanceof ImageAsset) {
            nativeImage = src.android;
        }
        else {
            nativeImage = src;
        }
        return nativeImage;
    }
    static _setFilter(filter, nativeView) {
        if (nativeView) {
            nativeView.setFilter(filter);
        }
    }
    [common.filterProperty.setNative](filter) {
        ImageCacheIt._setFilter(filter, this.nativeView);
    }
    [common.stretchProperty.getDefault]() {
        return 'aspectFit';
    }
    _setStretch(value) {
        if (this.nativeView) {
            switch (value) {
                case 'aspectFit':
                    this.nativeView.setScaleType(android.widget.ImageView.ScaleType.FIT_CENTER);
                    break;
                case 'aspectFill':
                    this.nativeView.setScaleType(android.widget.ImageView.ScaleType.CENTER_CROP);
                    break;
                case 'fill':
                    this.nativeView.setScaleType(android.widget.ImageView.ScaleType.FIT_XY);
                    break;
                case 'none':
                default:
                    this.nativeView.setScaleType(android.widget.ImageView.ScaleType.MATRIX);
                    break;
            }
        }
    }
    [common.stretchProperty.setNative](value) {
        this._setStretch(value);
    }
    static getItem(src) {
        com.github.triniwiz.imagecacheit.ImageCache.init(Application.android.context);
        return new Promise((resolve, reject) => {
            com.github.triniwiz.imagecacheit.ImageCache.getItem(src, null, new com.github.triniwiz.imagecacheit.ImageCache.Callback({
                onSuccess(value) {
                    resolve(value);
                },
                onError(error) {
                    reject(error.getMessage());
                },
            }));
        });
    }
    static deleteItem(src) {
        return new Promise((resolve, reject) => {
            // TODO
            resolve(undefined);
        });
    }
    static hasItem(src) {
        com.github.triniwiz.imagecacheit.ImageCache.init(Application.android.context);
        return new Promise((resolve, reject) => {
            com.github.triniwiz.imagecacheit.ImageCache.hasItem(src, new com.github.triniwiz.imagecacheit.ImageCache.Callback({
                onSuccess(value) {
                    resolve(undefined);
                },
                onError(error) {
                    reject(error.getMessage());
                },
            }));
        });
    }
    static clear() {
        com.github.triniwiz.imagecacheit.ImageCache.init(Application.android.context);
        return new Promise((resolve, reject) => {
            com.github.triniwiz.imagecacheit.ImageCache.clear();
            resolve(undefined);
        });
    }
    static enableAutoMM() {
        com.github.triniwiz.imagecacheit.ImageView.enableAutoMM(Application.android.nativeApp);
    }
    static disableAutoMM() {
        com.github.triniwiz.imagecacheit.ImageView.disableAutoMM(Application.android.nativeApp);
    }
}
//# sourceMappingURL=index.android.js.map